###############   PSQF 6249 FA & SEM Example 7c IFA Multiple-Group Invariance using WLSMV in R   #############

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=6) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("ggplot2")) install.packages("ggplot2")
library(ggplot2) # To make pictures

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs

# Other packages not used here, but which are pry useful

#if (!require("semTools")) install.packages("semTools")
#library(semTools) # lavaan add-ons (multiple imputation, moderation)

#if (!require("MplusAutomation")) install.packages("MplusAutomation")
#library(lavaan) # To use Mplus within R

#########################################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6249/OCTO/IRT Multiple-Group WLSMV/PSQF6249_Example7c/"
filename = "Example7cd.xlsx"
setwd(dir=filesave)

# Import OCTO IADL item-level data
Example7c = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame to use for analysis
Example7c = as.data.frame(Example7c)
# Recode sex to specify each group name (to show in output)
Example7c$female[which(Example7c$female==0)] = "0Men"
Example7c$female[which(Example7c$female==1)] = "1Women" 

# Create an object to hold categorical outcomes to use below
ciaItems = c("cia1", "cia2", "cia3", "cia4", "cia5", "cia6", "cia7")

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example7c_IFA_MultipleGroup_WLSMV_Output.txt"))

#### General notes about lavaan syntax objects:
# Once factors are defined they can be used in syntax as if they were observed variables
# Multiple commands can go on a single line separated by ; 
# Parameters can be fixed to constant values by using the * command
# Start values can be specified by using value* in front of parameter 
# Labels can be implemented with (label)* in front of parameter

# Example of how to concatenate syntax strings into one model
#Model <- sem(model=c(syntax1, syntax2), data = myData)

#########################################################################################################
# Measurement Invariance Model 1: Configural Invariance Model
# all parameters are estimated separately across groups except those for identification

ConfiguralSyntax = "
#===================================================================================================
# Factor loadings all freely estimated in both groups with label for each group
IADL =~ c(L1M, L1W)*cia1 + c(L2M, L2W)*cia2 + c(L3M, L3W)*cia3 + c(L4M, L4W)*cia4 +
        c(L5M, L5W)*cia5 + c(L6M, L6W)*cia6 + c(L7M, L7W)*cia7
#===================================================================================================
# Item thresholds all freely estimated in both groups with label for each group
cia1|c(I1T1M, I1T1W)*t1; cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3M, I1T3W)*t3
cia2|c(I2T1M, I2T1W)*t1; cia2|c(I2T2M, I2T2W)*t2; cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1M, I4T1W)*t1; cia4|c(I4T2M, I4T2W)*t2; cia4|c(I4T3M, I4T3W)*t3
cia5|c(I5T1M, I5T1W)*t1; cia5|c(I5T2M, I5T2W)*t2; cia5|c(I5T3M, I5T3W)*t3
cia6|c(I6T1M, I6T1W)*t1; cia6|c(I6T2M, I6T2W)*t2; cia6|c(I6T3M, I6T3W)*t3
cia7|c(I7T1M, I7T1W)*t1; cia7|c(I7T2M, I7T2W)*t2; cia7|c(I7T3M, I7T3W)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance fixed to 1 for identification in each group
IADL ~~ c(1,1)*IADL
#===================================================================================================
# Factor mean fixed to zero for identification in each group
IADL ~ c(0,0)*0
#===================================================================================================
"
Configural = lavaan(model=ConfiguralSyntax, data=Example7c, ordered=ciaItems, group="female",
                    parameterization="theta", mimic="mplus", missing="pairwise")
summary(Configural, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 2A: Metric Invariance Model
# constrain factor loadings equal across groups, estimate factor variance in alternative group

MetricASyntax = "
#===================================================================================================
# Factor loadings NOW CONSTRAINED EQUAL ACROSS GROUPS via same label
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3, L3)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds all freely estimated in both groups with label for each group
cia1|c(I1T1M, I1T1W)*t1; cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3M, I1T3W)*t3
cia2|c(I2T1M, I2T1W)*t1; cia2|c(I2T2M, I2T2W)*t2; cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1M, I4T1W)*t1; cia4|c(I4T2M, I4T2W)*t2; cia4|c(I4T3M, I4T3W)*t3
cia5|c(I5T1M, I5T1W)*t1; cia5|c(I5T2M, I5T2W)*t2; cia5|c(I5T3M, I5T3W)*t3
cia6|c(I6T1M, I6T1W)*t1; cia6|c(I6T2M, I6T2W)*t2; cia6|c(I6T3M, I6T3W)*t3
cia7|c(I7T1M, I7T1W)*t1; cia7|c(I7T2M, I7T2W)*t2; cia7|c(I7T3M, I7T3W)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but NOW ESTIMATED in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in each group
IADL ~ c(0,0)*0
#===================================================================================================
"
MetricA = lavaan(model=MetricASyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(MetricA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual loadings
modificationindices(object=MetricA, op="=~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is MetricA worse than Configural?")
anova(Configural, MetricA)

# Note: the results here indicate that full metric invariance holds, but to be consistent with the Mplus
# version of these models, I am freeing the loading for cia3 next

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 2B: Metric Invariance Model
# constrain factor loadings equal across groups, estimate factor variance in alternative group

MetricBSyntax = "
#===================================================================================================
# Factor loadings NOW CONSTRAINED EQUAL ACROSS GROUPS via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds all freely estimated in both groups with label for each group
cia1|c(I1T1M, I1T1W)*t1; cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3M, I1T3W)*t3
cia2|c(I2T1M, I2T1W)*t1; cia2|c(I2T2M, I2T2W)*t2; cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1M, I4T1W)*t1; cia4|c(I4T2M, I4T2W)*t2; cia4|c(I4T3M, I4T3W)*t3
cia5|c(I5T1M, I5T1W)*t1; cia5|c(I5T2M, I5T2W)*t2; cia5|c(I5T3M, I5T3W)*t3
cia6|c(I6T1M, I6T1W)*t1; cia6|c(I6T2M, I6T2W)*t2; cia6|c(I6T3M, I6T3W)*t3
cia7|c(I7T1M, I7T1W)*t1; cia7|c(I7T2M, I7T2W)*t2; cia7|c(I7T3M, I7T3W)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4 
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but NOW ESTIMATED in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in each group
IADL ~ c(0,0)*0
#===================================================================================================
"
MetricB = lavaan(model=MetricBSyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(MetricB, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual loadings
modificationindices(object=MetricB, op="=~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is MetricB better than MetricA?")
anova(MetricB, MetricA)

print("LRT: Is MetricB worse than Configural?")
anova(Configural, MetricB)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 3A: Scalar Invariance Model
# constrain thresholds equal across groups, estimate factor mean in alternative group

ScalarASyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL
cia1|c(I1T1, I1T1)*t1; cia1|c(I1T2, I1T2)*t2; cia1|c(I1T3, I1T3)*t3
cia2|c(I2T1, I2T1)*t1; cia2|c(I2T2, I2T2)*t2; cia2|c(I2T3, I2T3)*t3
cia3|c(I3T1, I3T1)*t1; cia3|c(I3T2, I3T2)*t2; cia3|c(I3T3, I3T3)*t3
cia4|c(I4T1, I4T1)*t1; cia4|c(I4T2, I4T2)*t2; cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1; cia5|c(I5T2, I5T2)*t2; cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1; cia6|c(I6T2, I6T2)*t2; cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1; cia7|c(I7T2, I7T2)*t2; cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but NOW ESTIMATED in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ScalarA = lavaan(model=ScalarASyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(ScalarA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ScalarA, op="|", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarA worse than MetricB?")
anova(MetricB, ScalarA)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 3B: Scalar Invariance Model (free cia3 thresholds)
# constrain thresholds equal across groups, estimate factor mean in alternative group

ScalarBSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2, I1T2)*t2;   cia1|c(I1T3, I1T3)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3, I2T3)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but NOW ESTIMATED in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ScalarB = lavaan(model=ScalarBSyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(ScalarB, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ScalarB, op="|", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarB worse than MetricB?")
anova(MetricB, ScalarB)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 3C: Scalar Invariance Model (free cia3 + cia1T3 thresholds)
# constrain thresholds equal across groups, estimate factor mean in alternative group

ScalarCSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3, cia1T3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2, I1T2)*t2;   cia1|c(I1T3C, I1T3W)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3, I2T3)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but NOW ESTIMATED in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ScalarC = lavaan(model=ScalarCSyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(ScalarC, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ScalarC, op="|", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarC worse than MetricB?")
anova(MetricB, ScalarC)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 3D: Scalar Invariance Model (free cia3 + cia1T3 + cia2T3 thresholds)
# constrain thresholds equal across groups, estimate factor mean in alternative group

ScalarDSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3, cia1T3, cia2T3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2, I1T2)*t2;   cia1|c(I1T3C, I1T3W)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but NOW ESTIMATED in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ScalarD = lavaan(model=ScalarDSyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(ScalarD, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ScalarD, op="|", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarD worse than MetricB?")
anova(MetricB, ScalarD)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 3E: Scalar Invariance Model (free cia3 + cia1T3 + cia2T3 + cia1T2 thresholds)
# constrain thresholds equal across groups, estimate factor mean in alternative group

ScalarESyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3, cia1T3, cia2T3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3C, I1T3W)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one for each group for identification
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,1)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6; cia7 ~~ c(1,1)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but NOW ESTIMATED in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ScalarE = lavaan(model=ScalarESyntax, data=Example7c, ordered=ciaItems, group="female",
                 parameterization="theta", mimic="mplus", missing="pairwise")
summary(ScalarE, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ScalarE, op="|", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarE worse than MetricB?")
anova(MetricB, ScalarE)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Measurement Invariance Model 4A: Free Residual Variance Model
# free previously constrained residual variances in alternative groups

ResidualFreeASyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3, cia1T3, cia2T3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3C, I1T3W)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one in ref group for identification, free in alt group (except 3)
cia1 ~~ c(1,NA)*cia1; cia2 ~~ c(1,NA)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,NA)*cia4
cia5 ~~ c(1,NA)*cia5; cia6 ~~ c(1,NA)*cia6; cia7 ~~ c(1,NA)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but still estimated in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ResidualFreeA = lavaan(model=ResidualFreeASyntax, data=Example7c, ordered=ciaItems, group="female",
                       parameterization="theta", mimic="mplus", missing="pairwise")
summary(ResidualFreeA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ResidualFreeA, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResidualFreeA better than ScalarE?")
anova(ResidualFreeA, ScalarE)

txtComment(" ") # insert blank space in output


#########################################################################################################
# NOTE: We don't need to estimate ResidualFixedB because lavaan model comparisons can be done in any
# order (unlike in Mplus, in which we had to save ScalarE as ResidualFixedB to do a model comparison)

# Measurement Invariance Model 4A: Partially Free Residual Variance Model (fixed except 2, 7)
# free previously constrained residual variances in alternative groups

ResidualPartialCSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3, cia1T3, cia2T3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3C, I1T3W)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one in ref group for identification, free in alt group (except 3)
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,NA)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6;  cia7 ~~ c(1,NA)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but still estimated in women 
IADL ~~ c(1,NA)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but still estimated in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
ResidualPartialC = lavaan(model=ResidualPartialCSyntax, data=Example7c, ordered=ciaItems, group="female",
                          parameterization="theta", mimic="mplus", missing="pairwise")
summary(ResidualPartialC, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

# Ask for mod indices to find potential non-invariance of individual thresholds
modificationindices(object=ResidualPartialC, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResidualPartialC worse than ResidualFreeA?")
anova(ResidualPartialC, ResidualFreeA)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Structural Invariance Model 5: Fix Factor Variance Equal (otherwise same as ResidualPartialC)

FactorVarSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label EXCEPT 3
IADL =~ c(L1, L1)*cia1 + c(L2, L2)*cia2 + c(L3M, L3W)*cia3 + c(L4, L4)*cia4 +
        c(L5, L5)*cia5 + c(L6, L6)*cia6 + c(L7, L7)*cia7
#===================================================================================================
# Item thresholds NOW CONSTRAINED EQUAL ACROSS GROUPS VIA SAME LABEL except cia3, cia1T3, cia2T3
cia1|c(I1T1, I1T1)*t1;   cia1|c(I1T2M, I1T2W)*t2; cia1|c(I1T3C, I1T3W)*t3
cia2|c(I2T1, I2T1)*t1;   cia2|c(I2T2, I2T2)*t2;   cia2|c(I2T3M, I2T3W)*t3
cia3|c(I3T1M, I3T1W)*t1; cia3|c(I3T2M, I3T2W)*t2; cia3|c(I3T3M, I3T3W)*t3
cia4|c(I4T1, I4T1)*t1;   cia4|c(I4T2, I4T2)*t2;   cia4|c(I4T3, I4T3)*t3
cia5|c(I5T1, I5T1)*t1;   cia5|c(I5T2, I5T2)*t2;   cia5|c(I5T3, I5T3)*t3
cia6|c(I6T1, I6T1)*t1;   cia6|c(I6T2, I6T2)*t2;   cia6|c(I6T3, I6T3)*t3
cia7|c(I7T1, I7T1)*t1;   cia7|c(I7T2, I7T2)*t2;   cia7|c(I7T3, I7T3)*t3
#===================================================================================================
# Residual variances all fixed to one in ref group for identification, free in alt group (except 3)
cia1 ~~ c(1,1)*cia1; cia2 ~~ c(1,NA)*cia2; cia3 ~~ c(1,1)*cia3; cia4 ~~ c(1,1)*cia4
cia5 ~~ c(1,1)*cia5; cia6 ~~ c(1,1)*cia6;  cia7 ~~ c(1,NA)*cia7
#===================================================================================================
# Factor variance still fixed to 1 for identification in men but  NOW FIXED TO 1 in women 
IADL ~~ c(1,1)*IADL
#===================================================================================================
# Factor mean still fixed to zero for identification in men but still estimated in women
IADL ~ c(0,NA)*0
#===================================================================================================
"
FactorVar = lavaan(model=FactorVarSyntax, data=Example7c, ordered=ciaItems, group="female",
                   parameterization="theta", mimic="mplus", missing="pairwise")
summary(FactorVar, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

print("LRT: Is FactorVar worse than ResidualPartialC?")
anova(ResidualPartialC, FactorVar)

txtComment(" ") # insert blank space in output

# The factor mean was significantly lower for women than mean already 
# (from the last partial measurement invariance model 4C), so we are done!

#########################################################################################################

# Close external file of saved results
txtStop()


